#!@l_prefix@/lib/openpkg/bash --noprofile -x
##
##  register -- OpenPKG Registry Command-Line Client
##  Copyright (c) 2000-2005 OpenPKG Foundation e.V. <http://openpkg.net/>
##  Copyright (c) 2000-2005 Ralf S. Engelschall <http://engelschall.com/>
##
##  Permission to use, copy, modify, and distribute this software for
##  any purpose with or without fee is hereby granted, provided that
##  the above copyright notice and this permission notice appear in all
##  copies.
##
##  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
##  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
##  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
##  IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS AND THEIR
##  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
##  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
##  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
##  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
##  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
##  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
##  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
##  SUCH DAMAGE.
##

##
##  configuration
##

#   program name, version and date
progname="register"
progvers="0.2.7"
progdate="23-Feb-2006"

#   determine path to OpenPKG instance
PREFIX="@l_prefix@"
if [ ".${OPENPKG_PREFIX}" != . ]; then
    PREFIX="${OPENPKG_PREFIX}"
fi

#   determine old binary-only vs. shell+binary situation
oldbinnewsh="$PREFIX/libexec/openpkg/rpm" # old binary rpm, new embedded shell rpm
oldnanewbin="$PREFIX/lib/openpkg/rpm"     # old n/a, new binary rpm
if [ -x $oldbinnewsh -a -x $oldnanewbin ]; then exec $oldnanewbin "$@"; fi

#   http post
curl="$PREFIX/lib/openpkg/curl"
useragent="openpkg-$progname/$progvers"

error()
{
    echo "$progname:ERROR: $@"
    exit 1
}

[ ".$PREFIX" = . ] && error "empty PREFIX not allowed"

##
##  read configuration unless already done during option parsing
##
sanitycheck()
{
    if [ ".$1" != . -a -r "$1" ]; then
        cat <"$1" | awk '
            BEGIN { rc=0 }
            !/^([A-Z][A-Z0-9_]+[A-Z0-9]="[^"]*")? *(#.*)?$/ { rc=1 }
            END { exit rc }' && return 0
    fi
    return 1
}

readconf()
{
    [ ".$REGISTRY_CONF" = . ] && return
    if [ -r "$REGISTRY_CONF" ]; then
        sanitycheck "$REGISTRY_CONF" && . "$REGISTRY_CONF"
    fi
}

readuuid()
{
    [ ".$REGISTRY_UUID" = . ] && return
    if [ -r "$REGISTRY_UUID" ]; then
        sanitycheck "$REGISTRY_UUID" && . "$REGISTRY_UUID"
    fi
}

readutil()
{
    [ ".$REGISTRY_UTIL" = . ] && return
    if [ -r "$REGISTRY_UTIL" ]; then
        sanitycheck "$REGISTRY_UTIL" && . "$REGISTRY_UTIL"
    fi
}

##
##  command line option parsing
##

#   default parameters

#   primary operations
op="automatic"

#   standard options
REGISTRY_MODE=""
REGISTRY_ARGS="http://registry.openpkg.org/register"
REGISTRY_USER=""
REGISTRY_DESC="openpkg://${HOSTNAME}${PREFIX}"

#   advanced options
REGISTRY_PLAT="`$oldbinnewsh --eval '%{l_platform -p}'`"
REGISTRY_OREL="`$oldbinnewsh --eval '%{l_openpkg_release}'`"
REGISTRY_UUID="$PREFIX/etc/openpkg/uuid"
REGISTRY_CONF="$PREFIX/etc/openpkg/register.conf"
REGISTRY_PREP="$PREFIX/etc/openpkg/register.prep"
REGISTRY_TRAN="$PREFIX/etc/openpkg/register.tran"
REGISTRY_UTIL="$PREFIX/etc/openpkg/register.util"

#   read baseline config early and allow options to override contents
readconf

#   debug options
verbose="no"
help="no"

#   iterate over argument line
declare -a a
declare -i i=0
while [ $# -gt 0 ]; do
    opt=$1
    case $opt in
        -*=*) arg=`echo "${opt}" | sed 's/^[-_a-zA-Z0-9]*=//'`
              opt=`echo "${opt}" | sed 's/=.*//'`
              ;;
           *) arg='' ;;
    esac
    case $opt in
        #   primary operations
        -P|--preparation ) op="preparation" ;;
        -T|--transaction ) op="transaction" ;;
        -U|--utilization ) op="utilization" ;;
        -C|--convenience ) op="convenience" ;;
        -I|--interaction ) op="interaction" ;;

        #   additional features
        -S|--printstatus ) op="printstatus" ;;
        -R|--rewriteurls ) op="rewriteurls" ;;

        #   standard options
        -m|--mode )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_MODE="$arg"
            ;;
        -a|--args )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_ARGS="$arg"
            ;;
        -u|--user )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_USER="$arg"
            ;;
        -d|--desc )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_DESC="$arg"
            ;;

        #   advanced options
        --plat )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_PLAT="$arg"
            ;;
        --orel )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_VERS="$arg"
            ;;
        --uuid )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_UUID="$arg"
            ;;
        --conf )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_CONF="$arg"; readconf
            ;;
        --prep )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_PREP="$arg"
            ;;
        --tran )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_TRAN="$arg"
            ;;
        --util )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_UTIL="$arg"
            ;;

        #   debug options
        -v|--verbose ) verbose="yes" ;;
        -h|--help    ) help="Usage"  ;;
        -*           ) help="Invalid option \`$opt'"; break ;;

        #   optional arguments
        * ) a[$i]="$opt"
            i=i+1
            ;;
    esac
    shift
done
set -- "${a[@]}"
unset i a

#   display error or usage message
if [ ".$help" != .no ]; then
    if [ ".$help" != ".Usage" ]; then
        echo "$progname:ERROR: $help" 1>&2
    fi
    echo "Usage: $progname -m|--mode=fake|post|wipe [-a|--args=<args>]"
    echo "       -u|--user=<user> [-d|--desc=<text>]"
    echo "       [--plat=<text>] [--orel=<text>] [--uuid=<file>]"
    echo "       [--conf=<file>] [--prep=<file>] [--tran=<file>] [--util=<file>]"
    echo "       [-P|--preparation] [-T|--transaction] [-U|--utilization]"
    echo "       [-C|--convenience] [-I|--interaction]"
    echo "       [-v|--verbose] [-h|--help]"
    echo "Usage: $progname -S|--printstatus"
    echo "Usage: $progname -R|--rewriteurls [[url]...]]"
    if [ ".$help" != ".Usage" ]; then
        exit 1
    else
        exit 0
    fi
fi

##
##  Primary operation Preparation
##
##  Creates registry data for one request and dumpes it stdout
##  in XML format. That request can be filtered and piped into
##  the --Tran phase or it can be copied and pasted into the
##  XMLdump CGI facility manually.
##
preparation()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        [ ".$REGISTRY_PREP" != . -a -w $REGISTRY_PREP ] && cat </dev/null >$REGISTRY_PREP
        return
    fi

    [ ".$REGISTRY_USER" = . ] && error "missing information REGISTRY_USER"
    [ ".$REGISTRY_DESC" = . ] && error "missing information REGISTRY_DESC"
    [ ".$REGISTRY_PLAT" = . ] && error "missing information REGISTRY_PLAT"
    [ ".$REGISTRY_OREL" = . ] && error "missing information REGISTRY_PREP"
    [ ".$REGISTRY_UUID" = . ] && error "missing information REGISTRY_UUID"
    [ ".$REGISTRY_PREP" = . ] && error "missing information REGISTRY_PREP"

    if [ -r "$REGISTRY_UUID" ]; then
        sanitycheck "$REGISTRY_UUID" && . "$REGISTRY_UUID"
    fi
    [ ".$UUID_REGISTRY" = . ] && error "missing information UUID_REGISTRY"
    [ ".$UUID_INSTANCE" = . ] && error "missing information UUID_INSTANCE"
    [ ".$UUID_PLATFORM" = . ] && error "missing information UUID_PLATFORM"

    (
        echo "<?xml version=\"1.0\" encoding=\"iso-8859-1\" standalone=\"no\"?>"
        echo "<!DOCTYPE registry"
        echo "  PUBLIC \"-//OpenPKG//DTD OpenPKG Registry 0.0.1//EN\""
        echo "  \"http://registry.openpkg.org/registry.dtd\" []>"
        echo "<registry>"
        echo "    <request id=\"$UUID_REGISTRY\""
        echo "        registry_user=\"$REGISTRY_USER\""
        echo "        registry_desc=\"$REGISTRY_DESC\""
        echo "        registry_plat=\"$REGISTRY_PLAT\""
        echo "        registry_orel=\"$REGISTRY_OREL\""
        echo "        uuid_registry=\"$UUID_REGISTRY\""
        echo "        uuid_instance=\"$UUID_INSTANCE\""
        echo "        uuid_platform=\"$UUID_PLATFORM\""
        echo "    />"
        echo "</registry>"
    ) | tee $REGISTRY_PREP
}

##
##  Primary operation Transaction
##
##  Reads registry data with one request from stdin, executes
##  the transaction and writes registry data with one or more
##  responses to stdout in XML format. Depending on the mode of
##  operation, the transaction might be an actual transport to the
##  XMLdump CGI facility or a fake activity which complements the
##  manual preparation.
##
transaction()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        [ ".$REGISTRY_TRAN" != . -a -w $REGISTRY_TRAN ] && cat </dev/null >$REGISTRY_TRAN
        return
    fi

    [ ".$REGISTRY_MODE" = . ] && error "missing information REGISTRY_MODE"
    [ ".$REGISTRY_ARGS" = . ] && error "missing information REGISTRY_ARGS"
    [ ".$REGISTRY_TRAN" = . ] && error "missing information REGISTRY_TRAN"

    if [ ".$REGISTRY_MODE" = .fake ]; then
        request=`awk '
            BEGIN { registry=0 }
            /<registry>/ { registry=1 }
            /<request / { if (registry) { print $0 } }
            /<\/registry>/ { registry=0 }' | \
            sed -e 's;^.*<request id=";;' -e 's;"$;;'`
        if [ ".$request" = . ]; then error "no request seen on stdin"; fi
        (
            echo "<?xml version=\"1.0\" encoding=\"iso-8859-1\" standalone=\"no\"?>"
            echo "<!DOCTYPE registry"
            echo "  PUBLIC \"-//OpenPKG//DTD OpenPKG Registry 0.0.1//EN\""
            echo "  \"http://registry.openpkg.org/registry.dtd\" []>"
            echo "<registry>"
            echo "    <response id=\"$request\" done=\"yes\">fake</response>"
            echo "</registry>"
        ) | tee $REGISTRY_TRAN
    fi

    if [ ".$REGISTRY_MODE" = .post ]; then
        $curl \
            --silent \
            --user-agent "$useragent" \
            --form 'page=dropxml' \
            --form 'data=<-' \
            --form 'submit=DROPXML' \
            $REGISTRY_ARGS \
        | tee $REGISTRY_TRAN
    fi
}

##
##  Primary operation Utilization
##
##  Reads registry data with one response from stdin and updates
##  the local registry information. Note that this step requires
##  write accesss to the $REGISTRY_UTIL file.
##
utilization()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        [ ".$REGISTRY_UTIL" != . -a -w $REGISTRY_UTIL ] && cat </dev/null >$REGISTRY_UTIL
        return
    fi

    [ ".$REGISTRY_UTIL" = . ] && error "missing information REGISTRY_UTIL"

    response=`awk '
        BEGIN { registry=0 }
        /<registry>/ { registry=1 }
        /<response / { if (registry) { print $0 } }
        /<\/registry>/ { registry=0 }' | \
        sed -e 's;^.*<response id=";;' -e 's;" done="; ;' -e 's;">; ;' -e 's;</response>.*$;;'`
    if [ ".$response" = . ]; then error "no response seen on stdin"; fi

    REGISTRY_UUID=`echo $response | awk '/./ { print $1 }'`
    REGISTRY_DONE=`echo $response | awk '/./ { print $2 }'`
    REGISTRY_RESP=`echo $response | sed -e 's;^[^ ][^ ]* [^ ][^ ]* ;;'`

    [ ".$REGISTRY_UUID" = . ] && error "missing information REGISTRY_UUID"
    [ ".$REGISTRY_DONE" = . ] && error "missing information REGISTRY_DONE"
    [ ".$REGISTRY_RESP" = . ] && error "missing information REGISTRY_RESP"

    (
        echo "REGISTRY_UUID=\"$REGISTRY_UUID\""
        echo "REGISTRY_DONE=\"$REGISTRY_DONE\""
        echo "REGISTRY_RESP=\"$REGISTRY_RESP\""
    ) | tee $REGISTRY_UTIL
}

##
##  Primary operation Convenience
##
convenience()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        utilization
        transaction
        preparation
        return
    fi

    preparation | transaction | utilization
}

##
##  Primary operation Interaction
##
interaction()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        [ ".$REGISTRY_CONF" != . -a -w $REGISTRY_CONF ] && cat </dev/null >$REGISTRY_CONF
        return
    fi

    echo "OpenPKG Registry Client ($useragent)"
    echo ""
    echo "Interactive step-by-step registration procedure."
    echo "You can abort at any time by just pressing CTRL-C."
    echo ""
    echo "Registration is a three step process:"
    echo "1. Preparation = create a configuration file and locally prepare a registration request"
    echo "2. Transaction = run a transaction to submit the request to the registry"
    echo "3. Utilization = memorize the response to activate local URL rewriting"
    echo ""
    echo "A (*) marks an optional field where the empty string \"\" is allowed."
    echo "Just press RETURN to continue and/or keep a default setting."

    echo ""
    echo "==== Step 1/3: Preparation ===="
    echo "Attempting to write to the configuration file \"$REGISTRY_CONF\""
    echo "#`TZ= date '+%Y-%m-%d %H:%M:%S'`" | tee -a "$REGISTRY_CONF" 2>/dev/null \
        || error "Configuration file not writable. Get more permissions and try again"

    read -p "[REGISTRY_USER=\"$REGISTRY_USER\"] username? ";          [ ".$REPLY" != . ] && REGISTRY_USER="$REPLY"
    read -p "[REGISTRY_DESC=\"$REGISTRY_DESC\"] description (*)? ";   [ ".$REPLY" != . ] && REGISTRY_DESC="$REPLY"
    read -p "[REGISTRY_PLAT=\"$REGISTRY_PLAT\"] platform (*)? ";      [ ".$REPLY" != . ] && REGISTRY_DESC="$REPLY"
    read -p "[REGISTRY_OREL=\"$REGISTRY_OREL\"] release (*)? ";       [ ".$REPLY" != . ] && REGISTRY_DESC="$REPLY"
    read -p "[REGISTRY_UUID=\"$REGISTRY_UUID\"] uuid file to read? "; [ ".$REPLY" != . ] && REGISTRY_UUID="$REPLY"
    read -p "[REGISTRY_PREP=\"$REGISTRY_PREP\"] prep dump file? ";    [ ".$REPLY" != . ] && REGISTRY_PREP="$REPLY"

    [ ".$REGISTRY_USER" = . ] && error "missing information REGISTRY_USER"
    [ ".$REGISTRY_DESC" = . ] && error "missing information REGISTRY_DESC"
    [ ".$REGISTRY_PLAT" = . ] && error "missing information REGISTRY_PLAT"
    [ ".$REGISTRY_OREL" = . ] && error "missing information REGISTRY_PREP"
    [ ".$REGISTRY_UUID" = . ] && error "missing information REGISTRY_UUID"
    [ ".$REGISTRY_PREP" = . ] && error "missing information REGISTRY_PREP"

    if [ -r "$REGISTRY_UUID" ]; then
        sanitycheck "$REGISTRY_UUID" && . "$REGISTRY_UUID"
    fi
    [ ".$UUID_REGISTRY" = . ] && error "missing information UUID_REGISTRY"
    [ ".$UUID_INSTANCE" = . ] && error "missing information UUID_INSTANCE"
    [ ".$UUID_PLATFORM" = . ] && error "missing information UUID_PLATFORM"

    echo "Appending to configuration file \"$REGISTRY_CONF\""
    (
        echo "REGISTRY_USER=\"$REGISTRY_USER\""
        echo "REGISTRY_DESC=\"$REGISTRY_DESC\""
        echo "REGISTRY_PLAT=\"$REGISTRY_PLAT\""
        echo "REGISTRY_OREL=\"$REGISTRY_OREL\""
        echo "REGISTRY_UUID=\"$REGISTRY_UUID\""
        echo "REGISTRY_PREP=\"$REGISTRY_PREP\""
    ) | tee -a "$REGISTRY_CONF"

    preparation

    echo ""
    echo "==== Step 2/3: Transaction ===="
    read -p "[REGISTRY_MODE=\"$REGISTRY_MODE\"] mode (post or fake)? "; [ ".$REPLY" != . ] && REGISTRY_MODE="$REPLY"
    read -p "[REGISTRY_ARGS=\"$REGISTRY_ARGS\"] args? ";                [ ".$REPLY" != . ] && REGISTRY_ARGS="$REPLY"
    read -p "[REGISTRY_TRAN=\"$REGISTRY_TRAN\"] tran dump file? ";      [ ".$REPLY" != . ] && REGISTRY_TRAN="$REPLY"

    [ ".$REGISTRY_MODE" = . ] && error "missing information REGISTRY_MODE"
    [ ".$REGISTRY_ARGS" = . ] && error "missing information REGISTRY_ARGS"
    [ ".$REGISTRY_TRAN" = . ] && error "missing information REGISTRY_TRAN"

    echo "Appending to configuration file \"$REGISTRY_CONF\""
    (
        echo "REGISTRY_MODE=\"$REGISTRY_MODE\""
        echo "REGISTRY_ARGS=\"$REGISTRY_ARGS\""
        echo "REGISTRY_TRAN=\"$REGISTRY_TRAN\""
    ) | tee -a "$REGISTRY_CONF"

    transaction <$REGISTRY_PREP

    echo ""
    echo "==== Step 3/3: Utilization ===="
    read -p "[REGISTRY_UTIL=\"$REGISTRY_UTIL\"] util dump file? ";      [ ".$REPLY" != . ] && REGISTRY_UTIL="$REPLY"

    [ ".$REGISTRY_UTIL" = . ] && error "missing information REGISTRY_UTIL"

    echo "Appending to configuration file \"$REGISTRY_CONF\""
    (
        echo "REGISTRY_UTIL=\"$REGISTRY_UTIL\""
    ) | tee -a "$REGISTRY_CONF"

    utilization <$REGISTRY_TRAN
}

##
##  Additional feature printstatus
##
printstatus()
{
    rc=0
    readconf || error "problem reading conf"
    readuuid || error "problem reading uuid"
    readutil || error "problem reading util"
    if [ ".$REGISTRY_DONE" = .yes ]; then
        cat "$REGISTRY_UTIL"
        return 0
    fi
    return 1
}

##
##  Additional feature rewriteurls
##
rewriteurls()
{
    readconf && readuuid && readutil || return $?
    if [ ".$REGISTRY_DONE" = .yes ]; then
        #   URL rewriting
        while [ ".$1" != . ]; do
            printf '%s\n' "$1" | \
            sed -e "s;^\(ftp\)://\(ftp.openpkg.com\)/;\1://$UUID_REGISTRY:$UUID_INSTANCE.$UUID_PLATFORM@\2/;" \
                -e "s;^\(ftp\)://\(ftp.openpkg.org\)/;\1://$UUID_REGISTRY:$UUID_INSTANCE.$UUID_PLATFORM@\2/;" \
                -e "s;^\(ftp\)://\(ftp.openpkg.net\)/;\1://$UUID_REGISTRY:$UUID_INSTANCE.$UUID_PLATFORM@\2/;"
            shift
        done
        return 0
    else
        #   stealth mode
        return 1
    fi
}

##
##  automatically pick interaction or convenience
##
automatic()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        convenience
        interaction
        return
    fi

    missing=""
    [ ".$REGISTRY_USER" = . ] && missing="$missing REGISTRY_USER"
    [ ".$REGISTRY_DESC" = . ] && missing="$missing REGISTRY_DESC"
    [ ".$REGISTRY_PLAT" = . ] && missing="$missing REGISTRY_PLAT"
    [ ".$REGISTRY_OREL" = . ] && missing="$missing REGISTRY_OREL"
    [ ".$REGISTRY_UUID" = . ] && missing="$missing REGISTRY_UUID"
    [ ".$REGISTRY_PREP" = . ] && missing="$missing REGISTRY_PREP"
    [ ".$REGISTRY_MODE" = . ] && missing="$missing REGISTRY_MODE"
    [ ".$REGISTRY_ARGS" = . ] && missing="$missing REGISTRY_ARGS"
    [ ".$REGISTRY_TRAN" = . ] && missing="$missing REGISTRY_TRAN"
    [ ".$REGISTRY_UTIL" = . ] && missing="$missing REGISTRY_UTIL"

    if [ ".$missing" = . ]; then
        convenience
    else
        echo "missing $missing"
        interaction
    fi
}

##
##  primary operation switch
##
eval $op "\"$@\""

